@@ -16,11 +16,13 @@ from kameleoon.configuration.variation import Variation
 from kameleoon.configuration.variation_by_exposition import VariationByExposition
 from kameleoon.helpers.visitor_code import validate_visitor_code
 from kameleoon.hybrid.hybrid_manager_impl import HybridManagerImpl
+from kameleoon.managers.warehouse.warehouse_manager import WarehouseManager
 
-from kameleoon.network.activity_event import ActivityEvent
 from kameleoon.data.manager.assigned_variation import AssignedVariation
 from kameleoon.data.manager.visitor import Visitor
 from kameleoon.data.manager.visitor_manager import VisitorManager
+from kameleoon.network.access_token_source_factory import AccessTokenSourceFactory
+from kameleoon.network.activity_event import ActivityEvent
 from kameleoon.network.sendable import Sendable
 from kameleoon.network.net_provider_impl import NetProviderImpl
 from kameleoon.network.net_provider import Response
@@ -126,9 +128,11 @@ class KameleoonClient:
         self._hybrid_manager = HybridManagerImpl(HYBRID_EXPIRATION_TIME)
 
         self._cookie_manager = CookieManager(config.top_level_domain)
+        atsf = AccessTokenSourceFactory(config.client_id, config.client_secret)
         self._network_manager = self._network_manager_factory.create(
-            self.site_code, config.environment, config.default_timeout_second, NetProviderImpl(), config.logger
+            self.site_code, config.environment, config.default_timeout_second, NetProviderImpl(), atsf, config.logger
         )
+        self._warehouse_manager = WarehouseManager(self._network_manager, self._visitor_manager, config.logger)
         self._thread_event_loop = ThreadEventLoop() if config.multi_threading else None
 
         self._timer: Optional[RepeatTimer] = self._scheduler.timer()
@@ -139,11 +143,14 @@ class KameleoonClient:
 
     def __del__(self):
         self._clear_timer()
-        if self._thread_event_loop:
-            self._thread_event_loop.run_coro(self._network_manager.net_provider.close())
-            self._thread_event_loop.stop()
-        else:
-            invoke_coro(self._network_manager.net_provider.close())
+        try:
+            if self._thread_event_loop:
+                self._thread_event_loop.run_coro(self._network_manager.net_provider.close())
+                self._thread_event_loop.stop()
+            else:
+                invoke_coro(self._network_manager.net_provider.close())
+        except AttributeError:
+            pass
 
     ###
     #   Public API methods
@@ -446,7 +453,8 @@ class KameleoonClient:
 
         :param key: key you want to retrieve data. This field is mandatory.
         :type key: str
-        :param timeout: requests timeout in seconds (default value is 5 seconds)
+        :param timeout: requests Timeout for request (in seconds). Equals default_timeout in a config file.
+        This field is optional.
         :type timeout: Optional[float]
 
         :return: data assosiated with this key, decoded into json
@@ -466,14 +474,17 @@ class KameleoonClient:
 
         :param key: key you want to retrieve data. This field is mandatory.
         :type key: str
-        :param timeout: requests timeout in seconds (default value is 5 seconds)
+        :param timeout: requests Timeout for request (in seconds). Equals default_timeout in a config file.
+        This field is optional.
         :type timeout: Optional[float]
 
         :return: data assosiated with this key, decoded into json
         :rtype: Optional[Any]
         """
-        coro = self.get_remote_data_async(key, timeout)
-        return self.__make_sync_call_anyway(coro, "get_remote_data")
+        service: DataService = self._network_manager.get_service(DataService)
+        coro = service.get_remote_data(key, timeout, sync=self.__is_sync_mode())
+        response = cast(Response, self.__make_sync_call_anyway(coro, "get_remote_data"))
+        return response.content
 
     async def get_remote_visitor_data_async(
         self, visitor_code: str, add_data=True, timeout: Optional[float] = None
@@ -484,11 +495,12 @@ class KameleoonClient:
         to the storage so that other methods could decide whether the current visitor is targeted or not.
 
         :param visitor_code: The visitor code for which you want to retrieve the assigned data. This field is mandatory.
-        :type key: str
+        :type visitor_code: str
         :param add_data: A boolean indicating whether the method should automatically add retrieved data for a visitor.
         If not specified, the default value is `True`. This field is optional.
         :type add_data: bool
-        :param timeout: requests timeout in seconds (default value is 5 seconds)
+        :param timeout: requests Timeout for request (in seconds). Equals default_timeout in a config file.
+        This field is optional.
         :type timeout: Optional[float]
 
         :return: A list of data assigned to the given visitor.
@@ -496,12 +508,7 @@ class KameleoonClient:
         """
         service: DataService = self._network_manager.get_service(DataService)
         response = await service.get_remote_visitor_data(visitor_code, timeout)
-        if response.content is None:
-            return []
-        data_list = self.__parse_custom_data_list(response.content)
-        if add_data:
-            self.add_data(visitor_code, *data_list)
-        return data_list
+        return self.__handle_remote_visitor_data_response(response, visitor_code, add_data)
 
     def get_remote_visitor_data(self, visitor_code: str, add_data=True, timeout: Optional[float] = None) -> List[Data]:
         """
@@ -510,21 +517,33 @@ class KameleoonClient:
         to the storage so that other methods could decide whether the current visitor is targeted or not.
 
         :param visitor_code: The visitor code for which you want to retrieve the assigned data. This field is mandatory.
-        :type key: str
+        :type visitor_code: str
         :param add_data: A boolean indicating whether the method should automatically add retrieved data for a visitor.
         If not specified, the default value is `True`. This field is optional.
         :type add_data: bool
-        :param timeout: requests timeout in seconds (default value is 5 seconds)
+        :param timeout: requests Timeout for request (in seconds). Equals default_timeout in a config file.
+        This field is optional.
         :type timeout: Optional[float]
 
         :return: A list of data assigned to the given visitor.
         :rtype: List[Data]
         """
-        coro = self.get_remote_visitor_data_async(visitor_code, add_data, timeout)
-        result = self.__make_sync_call_anyway(coro, "get_remote_visitor_data")
-        if result is None:
+        service: DataService = self._network_manager.get_service(DataService)
+        coro = service.get_remote_visitor_data(visitor_code, timeout, sync=self.__is_sync_mode())
+        response = self.__make_sync_call_anyway(coro, "get_remote_visitor_data")
+        if response is None:
             return []
-        return cast(List[Data], result)
+        return self.__handle_remote_visitor_data_response(response, visitor_code, add_data)
+
+    def __handle_remote_visitor_data_response(
+        self, response: Response, visitor_code: str, add_data: bool
+    ) -> List[Data]:
+        if response.content is None:
+            return []
+        data_list = self.__parse_custom_data_list(response.content)
+        if add_data:
+            self.add_data(visitor_code, *data_list)
+        return data_list
 
     def __parse_custom_data_list(self, raw: Any) -> List[Data]:
         try:
@@ -544,6 +563,87 @@ class KameleoonClient:
             self._config.logger.error(f"Parsing of visitor data failed: {ex}")
             return []
 
+    def get_visitor_warehouse_audience_async(
+        self,
+        visitor_code: str,
+        custom_data_index: int,
+        warehouse_key: Optional[str] = None,
+        timeout: Optional[float] = None,
+    ) -> Coroutine[Any, Any, Optional[CustomData]]:
+        """
+        Asynchronously retrieves data associated with a visitor's warehouse audiences and adds it to the visitor.
+        Retrieves all audience data associated with the visitor in your data warehouse using the specified
+        `visitor_code` and `warehouse_key`. The `warehouse_key` is typically your internal user ID.
+        The `custom_data_index` parameter corresponds to the Kameleoon custom data that Kameleoon uses to target your
+        visitors. You can refer to the
+        <a href="https://help.kameleoon.com/warehouse-audience-targeting/">warehouse targeting documentation</a>
+        for additional details. The method returns a `CustomData` object, confirming
+        that the data has been added to the visitor and is available for targeting purposes.
+
+        :param visitor_code: A unique visitor identification string, can't exceed 255 characters length.
+        This field is mandatory.
+        :type visitor_code: str
+        :param custom_data_index: An integer representing the index of the custom data you want to use to target
+        your BigQuery Audiences. This field is mandatory.
+        :type custom_data_index: int
+        :param warehouse_key: A key to identify the warehouse data, typically your internal user ID.
+        This field is optional.
+        :type warehouse_key: Optional[str]
+        :param timeout: requests Timeout for request (in seconds). Equals default_timeout in a config file.
+        This field is optional.
+        :type timeout: Optional[float]
+
+        :return: A `CustomData` instance confirming that the data has been added to the visitor.
+        :rtype: Optional[CustomData]
+
+        :raises:
+            VisitorCodeInvalid: Raise when the provided visitor code is not valid (empty, or longer than 255 characters)
+        """
+        return self._warehouse_manager.get_visitor_warehouse_audience(
+            visitor_code, custom_data_index, warehouse_key, timeout
+        )
+
+    def get_visitor_warehouse_audience(
+        self,
+        visitor_code: str,
+        custom_data_index: int,
+        warehouse_key: Optional[str] = None,
+        timeout: Optional[float] = None,
+    ) -> Optional[CustomData]:
+        """
+        Synchronously retrieves data associated with a visitor's warehouse audiences and adds it to the visitor.
+        Retrieves all audience data associated with the visitor in your data warehouse using the specified
+        `visitor_code` and `warehouse_key`. The `warehouse_key` is typically your internal user ID.
+        The `custom_data_index` parameter corresponds to the Kameleoon custom data that Kameleoon uses to target your
+        visitors. You can refer to the
+        <a href="https://help.kameleoon.com/warehouse-audience-targeting/">warehouse targeting documentation</a>
+        for additional details. The method returns a `CustomData` object, confirming
+        that the data has been added to the visitor and is available for targeting purposes.
+
+        :param visitor_code: A unique visitor identification string, can't exceed 255 characters length.
+        This field is mandatory.
+        :type visitor_code: str
+        :param custom_data_index: An integer representing the index of the custom data you want to use to target
+        your BigQuery Audiences. This field is mandatory.
+        :type custom_data_index: int
+        :param warehouse_key: A key to identify the warehouse data, typically your internal user ID.
+        This field is optional.
+        :type warehouse_key: Optional[str]
+        :param timeout: requests Timeout for request (in seconds). Equals default_timeout in a config file.
+        This field is optional.
+        :type timeout: Optional[float]
+
+        :return: A `CustomData` instance confirming that the data has been added to the visitor.
+        :rtype: Optional[CustomData]
+
+        :raises:
+            VisitorCodeInvalid: Raise when the provided visitor code is not valid (empty, or longer than 255 characters)
+        """
+        coro = self._warehouse_manager.get_visitor_warehouse_audience(
+            visitor_code, custom_data_index, warehouse_key, timeout, sync=self.__is_sync_mode()
+        )
+        return self.__make_sync_call_anyway(coro, "get_visitor_warehouse_audience")
+
     def get_feature_list(self) -> List[str]:
         """
         The get_feature_list method uses for obtaining a list of feature flag IDs:
@@ -748,7 +848,6 @@ class KameleoonClient:
             self._async_readiness.dispose_on_set()
 
         run_in_thread(initial_fetch, with_event_loop=True)
-        # run_in_thread(self._fetch_configuration, with_event_loop=True).join()  # //~ `join` was there all the time
 
     def _fetch_configuration(self, time_stamp: Optional[int] = None) -> bool:
         """
@@ -766,6 +865,7 @@ class KameleoonClient:
                 self._cookie_manager.consent_required = (
                     self._data_file.settings.is_consent_required and not self._data_file.has_any_targeted_delivery_rule
                 )
+                self._network_manager.url_provider.apply_data_api_domain(self._data_file.settings.data_api_domain)
                 self._call_update_handler_if_needed(time_stamp is not None)
             success = True
         except Exception as ex:
@@ -842,12 +942,12 @@ class KameleoonClient:
         Remove timer which updates configuration with specific interval (polling mode)
         :return: None
         """
-        if self._timer:
-            try:
+        try:
+            if self._timer:
                 self._timer.cancel()
                 self._timer = None
-            except AttributeError:
-                pass
+        except AttributeError:
+            pass
 
     def _obtain_configuration(self, time_stamp: Optional[int]) -> Optional[Dict[str, Any]]:
         """
