@@ -1,7 +1,6 @@
 """Experiment condition"""
 from typing import Any, Union, Dict
-
-
+from kameleoon.data.manager.assigned_variation import AssignedVariation
 from kameleoon.exceptions import NotFoundError
 from kameleoon.targeting.conditions.targeting_condition import TargetingCondition
 from kameleoon.targeting.conditions.constants import TargetingOperator
@@ -22,13 +21,13 @@ class TargetExperiment(TargetingCondition):
         except KeyError as ex:
             self._logger.error("%s has wrong JSON structure: %s", self.__class__, ex)
 
-    def check(self, data) -> bool:
-        is_targeted = False
-        variation_storage = dict[int, int](data)
-        is_saved_variation_storage_exist = bool(variation_storage)
+    def check(self, data: Any) -> bool:
+        return isinstance(data, dict) and self._check(data)
+
+    def _check(self, variation_storage: Dict[int, AssignedVariation]) -> bool:
         if self.__operator == TargetingOperator.EXACT:
-            saved_variation_value = variation_storage.get(self.experiment_id, 0)
-            is_targeted = is_saved_variation_storage_exist and saved_variation_value == self.__variation
-        elif self.__operator == TargetingOperator.ANY:
-            is_targeted = is_saved_variation_storage_exist
-        return is_targeted
+            as_variation = variation_storage.get(self.experiment_id)
+            return isinstance(as_variation, AssignedVariation) and (as_variation.variation_id == self.__variation)
+        if self.__operator == TargetingOperator.ANY:
+            return len(variation_storage) > 0
+        return False
